@@ -18,6 +18,8 @@
 #include <library.h>
 #include "fips_prf.h"
 
+static const char *plugin_name = "fips-prf";
+
 typedef struct private_fips_prf_plugin_t private_fips_prf_plugin_t;
 
 /**
@@ -31,10 +33,8 @@ struct private_fips_prf_plugin_t {
 	fips_prf_plugin_t public;
 };
 
-/**
- * Implementation of fips_prf_plugin_t.destroy
- */
-static void destroy(private_fips_prf_plugin_t *this)
+METHOD(plugin_t, destroy, void,
+	private_fips_prf_plugin_t *this)
 {
 	lib->crypto->remove_prf(lib->crypto,
 							(prf_constructor_t)fips_prf_create);
@@ -46,12 +46,24 @@ static void destroy(private_fips_prf_plugin_t *this)
  */
 plugin_t *fips_prf_plugin_create()
 {
-	private_fips_prf_plugin_t *this = malloc_thing(private_fips_prf_plugin_t);
+	private_fips_prf_plugin_t *this;
+	prf_t *prf;
 
-	this->public.plugin.destroy = (void(*)(plugin_t*))destroy;
+	INIT(this,
+		.public = {
+			.plugin = {
+				.destroy = _destroy,
+			},
+		},
+	);
 
-	lib->crypto->add_prf(lib->crypto, PRF_FIPS_SHA1_160,
-						 (prf_constructor_t)fips_prf_create);
+	prf = lib->crypto->create_prf(lib->crypto, PRF_KEYED_SHA1);
+	if (prf)
+	{
+		prf->destroy(prf);
+		lib->crypto->add_prf(lib->crypto, PRF_FIPS_SHA1_160, plugin_name,
+							 (prf_constructor_t)fips_prf_create);
+	}
 
 	return &this->public.plugin;
 }
