@@ -31,10 +31,8 @@ struct private_sqlite_plugin_t {
 	sqlite_plugin_t public;
 };
 
-/**
- * Implementation of plugin_t.destroy
- */
-static void destroy(private_sqlite_plugin_t *this)
+METHOD(plugin_t, destroy, void,
+	private_sqlite_plugin_t *this)
 {
 	lib->db->remove_database(lib->db,
 							 (database_constructor_t)sqlite_database_create);
@@ -46,9 +44,15 @@ static void destroy(private_sqlite_plugin_t *this)
  */
 plugin_t *sqlite_plugin_create()
 {
-	private_sqlite_plugin_t *this = malloc_thing(private_sqlite_plugin_t);
+	private_sqlite_plugin_t *this;
 
-	this->public.plugin.destroy = (void(*)(plugin_t*))destroy;
+	INIT(this,
+		.public = {
+			.plugin = {
+				.destroy = _destroy,
+			},
+		},
+	);
 
 	lib->db->add_database(lib->db,
 						  (database_constructor_t)sqlite_database_create);
